# Copyright © 2022 IOTIC LABS LTD. info@iotics.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://github.com/Iotic-Labs/iotics-grpc-client-py/blob/main/LICENSE
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import typing

import grpc
import grpc._channel

from iotics.api.input_pb2 import InputID

from .base import ApiBase
from .helpers import create_headers
from iotics.api import input_pb2_grpc
from iotics.api import input_pb2
from iotics.api import common_pb2


class InputApi(ApiBase):
    stub_class = input_pb2_grpc.InputAPIStub
    stub: input_pb2_grpc.InputAPIStub

    def create_input(
            self, twin_did: str,
            input_id: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> input_pb2.CreateInputResponse:
        """Create a input on the given twin
        Args:
            twin_did: Decentralized Identifier of the twin on which a input should be created
            input_id: A string identifying this input that is unique to this twin
            headers: optional request headers
        Returns: Response object confirming the input's details (ie, the two IDs that were given)
        """
        req = input_pb2.CreateInputRequest(
            headers=headers or create_headers(),
            args=input_pb2.CreateInputRequest.Arguments(twinId=common_pb2.TwinID(id=twin_did)),
            payload=input_pb2.CreateInputRequest.Payload(id=input_id)
        )
        return self.stub.CreateInput(req)

    def update_input(
            self,
            twin_did: str,
            input_id: str,
            values_added: typing.Optional[list] = None,
            values_deleted: typing.Optional[list] = None,
            props_added: typing.Optional[list] = None,
            props_deleted: typing.Optional[list] = None,
            props_keys_deleted: typing.Optional[list] = None,
            clear_all_props: bool = False,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> input_pb2.UpdateInputResponse:
        """Update the details of the given input. Any arguments omitted will have their values unchanged
        Args:
            twin_did: Decentralized Identifier of the twin providing this input
            input_id: A string identifying this input that is unique to this twin
            values_added: A list of Value objects (metadata describing the structure of shared data) to be added to the
                input. These may be created via the create_value helper
            values_deleted: A list of Value objects to be removed from the input
            props_added: A list of semantic properties to be added to the input
            props_deleted: A list of semantic properties to be removed from the input
            props_keys_deleted: A list of any property keys for which all values should be removed from the input
            clear_all_props: Whether or not to remove all semantic properties from the input
            headers: optional request headers
        Returns: Response object confirming the IDs of the twin and input being updated
        """
        req = input_pb2.UpdateInputRequest(
            headers=headers or create_headers(),
            args=input_pb2.UpdateInputRequest.Arguments(
                inputId=input_pb2.InputID(id=input_id, twinId=twin_did)
            ),
            payload=input_pb2.UpdateInputRequest.Payload(
                values=common_pb2.Values(added=values_added, deletedByLabel=values_deleted),
                properties=common_pb2.PropertyUpdate(
                    added=props_added, deleted=props_deleted, deletedByKey=props_keys_deleted,
                    clearedAll=clear_all_props
                )
            )
        )
        return self.stub.UpdateInput(req)


    def describe_input(
        self,
        twin_did: str,
        input_id: str,
        remote_host_id: typing.Optional[str] = None,
        headers: typing.Optional[common_pb2.Headers] = None,
    ) -> input_pb2.DescribeInputResponse:
        """Describes an input (local and remote).

        Args:
            twin_did: The twin whose input will be described
            input_id: The ID of the input to be described
            remote_host_id: The remote host on which the twin is found -- None if twin is local
            headers: optional request headers

        Returns: A response object describing the input.
        """
        request = input_pb2.DescribeInputRequest(
            headers=headers or create_headers(),
            args=input_pb2.DescribeInputRequest.Arguments(
                inputId=InputID(id=input_id, twinId=twin_did, hostId=remote_host_id),
            )
        )
        return self.stub.DescribeInput(request)

    def delete_input(
        self,
        twin_did: str,
        input_id: str,
        headers: typing.Optional[common_pb2.Headers] = None,
    ) -> input_pb2.DeleteInputResponse:
        """Deletes an input (idempotent).

        Args:
            twin_did: The twin whose input will be deleted
            input_id: The ID of the input to be deleted
            headers: optional request headers

        Returns: A response object with a payload containing the deleted input.

        """
        request = input_pb2.DeleteInputRequest(
            headers=headers or create_headers(),
            args=input_pb2.DeleteInputRequest.Arguments(inputId=InputID(id=input_id, twinId=twin_did)),
        )
        return self.stub.DeleteInput(request)

    def receive_input_messages(
            self,
            twin_did: str,
            input_id: str,
            timeout: typing.Optional[int] = None,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> typing.Iterator[input_pb2.ReceiveInputMessageResponse]:
        """Initialises an iterator that listens for input messages for a given Twin ID and Input ID.

        Note: This function must be called before `InterestApi.send_input_message` otherwise messages may be lost.

        Args:
            twin_did: The twin offering the input to receive messages on
            input_id: The input that will receive the messages
            timeout: How long before the input stops listening
            headers: optional request headers

        Returns: Response iterator with extra blocking (e.g. `time_remaining`) and non-blocking (e.g. `code`) methods.
        """
        args = input_pb2.ReceiveInputMessageRequest.Arguments(inputId=InputID(id=input_id, twinId=twin_did))
        request = input_pb2.ReceiveInputMessageRequest(headers=headers or create_headers(), args=args)

        # Define type here to avoid: `TypeError: 'ABCMeta' object is not subscriptable`.
        input_responses: grpc._channel._MultiThreadedRendezvous[input_pb2.ReceiveInputMessageResponse] = \
            self.stub.ReceiveInputMessages(request, timeout=timeout)
        return input_responses



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from iotics.api import meta_pb2 as iotics_dot_api_dot_meta__pb2


class MetaAPIStub(object):
    """---------------------------------------------------------------------------------------------------------------------

    MetaAPI enables querying of metadata associated with Twins and Feeds.
    Services only affect local resources, unless stated otherwise.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SparqlQuery = channel.unary_stream(
                '/iotics.api.MetaAPI/SparqlQuery',
                request_serializer=iotics_dot_api_dot_meta__pb2.SparqlQueryRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_meta__pb2.SparqlQueryResponse.FromString,
                _registered_method=True)
        self.SparqlUpdate = channel.unary_unary(
                '/iotics.api.MetaAPI/SparqlUpdate',
                request_serializer=iotics_dot_api_dot_meta__pb2.SparqlUpdateRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_meta__pb2.SparqlUpdateResponse.FromString,
                _registered_method=True)


class MetaAPIServicer(object):
    """---------------------------------------------------------------------------------------------------------------------

    MetaAPI enables querying of metadata associated with Twins and Feeds.
    Services only affect local resources, unless stated otherwise.
    """

    def SparqlQuery(self, request, context):
        """SparqlQuery performs a SPARQL 1.1 query against the Federated Knowledge Graph of the Iotics network to which this
        host belongs. The result is returned as a sequence of chunks. Note that:
        - Result chunks MIGHT arrive out of order and it is the client's responsibility to re-assemble them.
        - This RPC is currently in beta, it means:
        - the logic should remain unchanged,
        - the rpc call should remain unchanged,
        - the service can be interrupted without notice.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SparqlUpdate(self, request, context):
        """SparqlUpdate performs a SPARQL 1.1 update. When performing an update, the update query must contain a reference to
        one of the following graph IRIs:
        1. http://data.iotics.com/graph#custom-public (aka custom public graph) - All metadata written to this graph will be
        visible during SPARQL queries both with local & global scope (and thus, the Iotics network).
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MetaAPIServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SparqlQuery': grpc.unary_stream_rpc_method_handler(
                    servicer.SparqlQuery,
                    request_deserializer=iotics_dot_api_dot_meta__pb2.SparqlQueryRequest.FromString,
                    response_serializer=iotics_dot_api_dot_meta__pb2.SparqlQueryResponse.SerializeToString,
            ),
            'SparqlUpdate': grpc.unary_unary_rpc_method_handler(
                    servicer.SparqlUpdate,
                    request_deserializer=iotics_dot_api_dot_meta__pb2.SparqlUpdateRequest.FromString,
                    response_serializer=iotics_dot_api_dot_meta__pb2.SparqlUpdateResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'iotics.api.MetaAPI', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('iotics.api.MetaAPI', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class MetaAPI(object):
    """---------------------------------------------------------------------------------------------------------------------

    MetaAPI enables querying of metadata associated with Twins and Feeds.
    Services only affect local resources, unless stated otherwise.
    """

    @staticmethod
    def SparqlQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/iotics.api.MetaAPI/SparqlQuery',
            iotics_dot_api_dot_meta__pb2.SparqlQueryRequest.SerializeToString,
            iotics_dot_api_dot_meta__pb2.SparqlQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SparqlUpdate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.MetaAPI/SparqlUpdate',
            iotics_dot_api_dot_meta__pb2.SparqlUpdateRequest.SerializeToString,
            iotics_dot_api_dot_meta__pb2.SparqlUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: iotics/api/circle.proto
# Protobuf Python Version: 5.27.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    1,
    '',
    'iotics/api/circle.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from iotics.api import common_pb2 as iotics_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17iotics/api/circle.proto\x12\niotics.api\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17iotics/api/common.proto\"\x1a\n\x08\x43ircleID\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\xfa\x01\n\x13UpsertCircleRequest\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x41\n\x07payload\x18\x02 \x01(\x0b\x32\'.iotics.api.UpsertCircleRequest.PayloadR\x07payload\x1aq\n\x07Payload\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\x12\x34\n\nproperties\x18\x02 \x03(\x0b\x32\x14.iotics.api.PropertyR\nproperties\"\xc6\x01\n\x14UpsertCircleResponse\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x42\n\x07payload\x18\x02 \x01(\x0b\x32(.iotics.api.UpsertCircleResponse.PayloadR\x07payload\x1a;\n\x07Payload\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\"\xc2\x01\n\x13\x44\x65leteCircleRequest\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12=\n\x04\x61rgs\x18\x02 \x01(\x0b\x32).iotics.api.DeleteCircleRequest.ArgumentsR\x04\x61rgs\x1a=\n\tArguments\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\"\xc6\x01\n\x14\x44\x65leteCircleResponse\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x42\n\x07payload\x18\x02 \x01(\x0b\x32(.iotics.api.DeleteCircleResponse.PayloadR\x07payload\x1a;\n\x07Payload\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\"\xc6\x01\n\x15\x44\x65scribeCircleRequest\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12?\n\x04\x61rgs\x18\x03 \x01(\x0b\x32+.iotics.api.DescribeCircleRequest.ArgumentsR\x04\x61rgs\x1a=\n\tArguments\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\"\x90\x03\n\x16\x44\x65scribeCircleResponse\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x44\n\x07payload\x18\x02 \x01(\x0b\x32*.iotics.api.DescribeCircleResponse.PayloadR\x07payload\x1a|\n\nMetaResult\x12\x34\n\nproperties\x18\x01 \x03(\x0b\x32\x14.iotics.api.PropertyR\nproperties\x12\x38\n\tupdatedAt\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x1a\x82\x01\n\x07Payload\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\x12\x45\n\x06result\x18\x02 \x01(\x0b\x32-.iotics.api.DescribeCircleResponse.MetaResultR\x06result\"o\n\x15ListAllCirclesRequest\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\'\n\x05range\x18\x14 \x01(\x0b\x32\x11.iotics.api.RangeR\x05range\"\x98\x03\n\x16ListAllCirclesResponse\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x44\n\x07payload\x18\x02 \x01(\x0b\x32*.iotics.api.ListAllCirclesResponse.PayloadR\x07payload\x1a\xb1\x01\n\rCircleDetails\x12\x30\n\x08\x63ircleId\x18\x01 \x01(\x0b\x32\x14.iotics.api.CircleIDR\x08\x63ircleId\x12\x34\n\nproperties\x18\x02 \x03(\x0b\x32\x14.iotics.api.PropertyR\nproperties\x12\x38\n\tupdatedAt\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x1aU\n\x07Payload\x12J\n\x07\x63ircles\x18\x01 \x03(\x0b\x32\x30.iotics.api.ListAllCirclesResponse.CircleDetailsR\x07\x63ircles2\xeb\x02\n\tCircleAPI\x12S\n\x0cUpsertCircle\x12\x1f.iotics.api.UpsertCircleRequest\x1a .iotics.api.UpsertCircleResponse\"\x00\x12S\n\x0c\x44\x65leteCircle\x12\x1f.iotics.api.DeleteCircleRequest\x1a .iotics.api.DeleteCircleResponse\"\x00\x12Y\n\x0e\x44\x65scribeCircle\x12!.iotics.api.DescribeCircleRequest\x1a\".iotics.api.DescribeCircleResponse\"\x00\x12Y\n\x0eListAllCircles\x12!.iotics.api.ListAllCirclesRequest\x1a\".iotics.api.ListAllCirclesResponse\"\x00\x42\x7f\n\x0e\x63om.iotics.apiB\x0b\x43ircleProtoP\x01Z>github.com/Iotic-Labs/iotic-go-proto-qapi/iotics/api;ioticsapi\xa2\x02\x03IAX\xaa\x02\nIotics.Api\xca\x02\nIotics\\Apib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'iotics.api.circle_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016com.iotics.apiB\013CircleProtoP\001Z>github.com/Iotic-Labs/iotic-go-proto-qapi/iotics/api;ioticsapi\242\002\003IAX\252\002\nIotics.Api\312\002\nIotics\\Api'
  _globals['_CIRCLEID']._serialized_start=97
  _globals['_CIRCLEID']._serialized_end=123
  _globals['_UPSERTCIRCLEREQUEST']._serialized_start=126
  _globals['_UPSERTCIRCLEREQUEST']._serialized_end=376
  _globals['_UPSERTCIRCLEREQUEST_PAYLOAD']._serialized_start=263
  _globals['_UPSERTCIRCLEREQUEST_PAYLOAD']._serialized_end=376
  _globals['_UPSERTCIRCLERESPONSE']._serialized_start=379
  _globals['_UPSERTCIRCLERESPONSE']._serialized_end=577
  _globals['_UPSERTCIRCLERESPONSE_PAYLOAD']._serialized_start=263
  _globals['_UPSERTCIRCLERESPONSE_PAYLOAD']._serialized_end=322
  _globals['_DELETECIRCLEREQUEST']._serialized_start=580
  _globals['_DELETECIRCLEREQUEST']._serialized_end=774
  _globals['_DELETECIRCLEREQUEST_ARGUMENTS']._serialized_start=713
  _globals['_DELETECIRCLEREQUEST_ARGUMENTS']._serialized_end=774
  _globals['_DELETECIRCLERESPONSE']._serialized_start=777
  _globals['_DELETECIRCLERESPONSE']._serialized_end=975
  _globals['_DELETECIRCLERESPONSE_PAYLOAD']._serialized_start=263
  _globals['_DELETECIRCLERESPONSE_PAYLOAD']._serialized_end=322
  _globals['_DESCRIBECIRCLEREQUEST']._serialized_start=978
  _globals['_DESCRIBECIRCLEREQUEST']._serialized_end=1176
  _globals['_DESCRIBECIRCLEREQUEST_ARGUMENTS']._serialized_start=713
  _globals['_DESCRIBECIRCLEREQUEST_ARGUMENTS']._serialized_end=774
  _globals['_DESCRIBECIRCLERESPONSE']._serialized_start=1179
  _globals['_DESCRIBECIRCLERESPONSE']._serialized_end=1579
  _globals['_DESCRIBECIRCLERESPONSE_METARESULT']._serialized_start=1322
  _globals['_DESCRIBECIRCLERESPONSE_METARESULT']._serialized_end=1446
  _globals['_DESCRIBECIRCLERESPONSE_PAYLOAD']._serialized_start=1449
  _globals['_DESCRIBECIRCLERESPONSE_PAYLOAD']._serialized_end=1579
  _globals['_LISTALLCIRCLESREQUEST']._serialized_start=1581
  _globals['_LISTALLCIRCLESREQUEST']._serialized_end=1692
  _globals['_LISTALLCIRCLESRESPONSE']._serialized_start=1695
  _globals['_LISTALLCIRCLESRESPONSE']._serialized_end=2103
  _globals['_LISTALLCIRCLESRESPONSE_CIRCLEDETAILS']._serialized_start=1839
  _globals['_LISTALLCIRCLESRESPONSE_CIRCLEDETAILS']._serialized_end=2016
  _globals['_LISTALLCIRCLESRESPONSE_PAYLOAD']._serialized_start=2018
  _globals['_LISTALLCIRCLESRESPONSE_PAYLOAD']._serialized_end=2103
  _globals['_CIRCLEAPI']._serialized_start=2106
  _globals['_CIRCLEAPI']._serialized_end=2469
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright (c) 2019-2024 Iotic Labs Ltd. All rights reserved.

Iotics Web protocol definitions (hosts)
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import iotics.api.common_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class GetHostIDRequest(google.protobuf.message.Message):
    """GetHostIDRequest gets the local host twin's ID"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEADERS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers: ...
    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers"]) -> None: ...

global___GetHostIDRequest = GetHostIDRequest

@typing.final
class GetHostIDResponse(google.protobuf.message.Message):
    """GetHostIDResponse is the response containing the local host twin's ID"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        HOSTID_FIELD_NUMBER: builtins.int
        hostId: builtins.str
        def __init__(
            self,
            *,
            hostId: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["hostId", b"hostId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers: ...
    @property
    def payload(self) -> global___GetHostIDResponse.Payload: ...
    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___GetHostIDResponse.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___GetHostIDResponse = GetHostIDResponse

@typing.final
class HostAllowedArguments(google.protobuf.message.Message):
    """HostAllowedArguments are the arguments determining which host and twin the allow list check applies to."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REMOTEHOSTID_FIELD_NUMBER: builtins.int
    TWINID_FIELD_NUMBER: builtins.int
    remoteHostId: builtins.str
    """Remote Host Id"""
    @property
    def twinId(self) -> iotics.api.common_pb2.TwinID:
        """TwinDID of the twin to which the request applies"""

    def __init__(
        self,
        *,
        remoteHostId: builtins.str = ...,
        twinId: iotics.api.common_pb2.TwinID | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["twinId", b"twinId"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["remoteHostId", b"remoteHostId", "twinId", b"twinId"]) -> None: ...

global___HostAllowedArguments = HostAllowedArguments

@typing.final
class HostAllowedResponsePayload(google.protobuf.message.Message):
    """HostAllowedResponsePayload contans response details to a data/meta allow list check."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REMOTEHOSTID_FIELD_NUMBER: builtins.int
    TWINID_FIELD_NUMBER: builtins.int
    ALLOWED_FIELD_NUMBER: builtins.int
    remoteHostId: builtins.str
    """Remote Host Id"""
    allowed: builtins.bool
    """Whether the twin allows the remote host to perform meta or data actions"""
    @property
    def twinId(self) -> iotics.api.common_pb2.TwinID:
        """TwinDID of the twin to which the request applies"""

    def __init__(
        self,
        *,
        remoteHostId: builtins.str = ...,
        twinId: iotics.api.common_pb2.TwinID | None = ...,
        allowed: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["twinId", b"twinId"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["allowed", b"allowed", "remoteHostId", b"remoteHostId", "twinId", b"twinId"]) -> None: ...

global___HostAllowedResponsePayload = HostAllowedResponsePayload

@typing.final
class IsHostDataAllowedRequest(google.protobuf.message.Message):
    """IsHostDataAllowedRequest determines whether a remote host is allowed to perform data requests against the given twin."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """IsHostDataAllowedRequest headers"""

    @property
    def args(self) -> global___HostAllowedArguments:
        """IsHostDataAllowedRequest mandatory arguments"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___HostAllowedArguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> None: ...

global___IsHostDataAllowedRequest = IsHostDataAllowedRequest

@typing.final
class IsHostDataAllowedResponse(google.protobuf.message.Message):
    """IsHostDataAllowedResponse is the response to for an IsHostDataAllowed call"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """IsHostDataAllowedResponse headers"""

    @property
    def payload(self) -> global___HostAllowedResponsePayload:
        """IsHostDataAllowedResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___HostAllowedResponsePayload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___IsHostDataAllowedResponse = IsHostDataAllowedResponse

@typing.final
class IsHostMetaAllowedRequest(google.protobuf.message.Message):
    """IsHostMetaAllowedRequest determines whether a remote host is allowed to perform meta requests against the given twin."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """IsHostMetaAllowedRequest headers"""

    @property
    def args(self) -> global___HostAllowedArguments:
        """IsHostMetaAllowedRequest mandatory arguments"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___HostAllowedArguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> None: ...

global___IsHostMetaAllowedRequest = IsHostMetaAllowedRequest

@typing.final
class IsHostMetaAllowedResponse(google.protobuf.message.Message):
    """IsHostDataAllowedResponse is the response to for an IsHostMetaAllowed call"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """IsHostMetaAllowedResponse headers"""

    @property
    def payload(self) -> global___HostAllowedResponsePayload:
        """IsHostMetaAllowedResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___HostAllowedResponsePayload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___IsHostMetaAllowedResponse = IsHostMetaAllowedResponse



# Copyright © 2022 IOTIC LABS LTD. info@iotics.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://github.com/Iotic-Labs/iotics-grpc-client-py/blob/main/LICENSE
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import typing

import grpc

from iotics.api import common_pb2, input_pb2
from iotics.lib.grpc.auth import AuthInterface
from iotics.lib.grpc.helpers import get_channel


class ApiBase:
    stub_class = None

    def __init__(self, auth: AuthInterface, channel: typing.Optional[grpc.Channel] = None):
        channel = channel or get_channel(auth)
        self.stub = self.stub_class(channel)
        self.address = auth.get_host()
        self.token = auth.get_token()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: iotics/api/meta.proto
# Protobuf Python Version: 5.27.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    1,
    '',
    'iotics/api/meta.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2
from iotics.api import common_pb2 as iotics_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15iotics/api/meta.proto\x12\niotics.api\x1a\x17google/rpc/status.proto\x1a\x17iotics/api/common.proto\"\x9b\x02\n\x12SparqlQueryRequest\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\'\n\x05scope\x18\x02 \x01(\x0e\x32\x11.iotics.api.ScopeR\x05scope\x12@\n\x07payload\x18\x03 \x01(\x0b\x32&.iotics.api.SparqlQueryRequest.PayloadR\x07payload\x1ak\n\x07Payload\x12J\n\x11resultContentType\x18\x01 \x01(\x0e\x32\x1c.iotics.api.SparqlResultTypeR\x11resultContentType\x12\x14\n\x05query\x18\x02 \x01(\x0cR\x05query\"\xcd\x02\n\x13SparqlQueryResponse\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x41\n\x07payload\x18\x02 \x01(\x0b\x32\'.iotics.api.SparqlQueryResponse.PayloadR\x07payload\x1a\xc3\x01\n\x07Payload\x12\x16\n\x06seqNum\x18\x01 \x01(\x04R\x06seqNum\x12\x12\n\x04last\x18\x02 \x01(\x08R\x04last\x12*\n\x06status\x18\x03 \x01(\x0b\x32\x12.google.rpc.StatusR\x06status\x12>\n\x0b\x63ontentType\x18\x04 \x01(\x0e\x32\x1c.iotics.api.SparqlResultTypeR\x0b\x63ontentType\x12 \n\x0bresultChunk\x18\x05 \x01(\x0cR\x0bresultChunk\"\xaa\x01\n\x13SparqlUpdateRequest\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers\x12\x41\n\x07payload\x18\x02 \x01(\x0b\x32\'.iotics.api.SparqlUpdateRequest.PayloadR\x07payload\x1a!\n\x07Payload\x12\x16\n\x06update\x18\x01 \x01(\x0cR\x06update\"E\n\x14SparqlUpdateResponse\x12-\n\x07headers\x18\x01 \x01(\x0b\x32\x13.iotics.api.HeadersR\x07headers*r\n\x10SparqlResultType\x12\x0f\n\x0bSPARQL_JSON\x10\x00\x12\x0e\n\nSPARQL_XML\x10\x01\x12\x0e\n\nSPARQL_CSV\x10\x02\x12\x0e\n\nRDF_TURTLE\x10\x03\x12\x0b\n\x07RDF_XML\x10\x04\x12\x10\n\x0cRDF_NTRIPLES\x10\x05\x32\xb2\x01\n\x07MetaAPI\x12R\n\x0bSparqlQuery\x12\x1e.iotics.api.SparqlQueryRequest\x1a\x1f.iotics.api.SparqlQueryResponse\"\x00\x30\x01\x12S\n\x0cSparqlUpdate\x12\x1f.iotics.api.SparqlUpdateRequest\x1a .iotics.api.SparqlUpdateResponse\"\x00\x42}\n\x0e\x63om.iotics.apiB\tMetaProtoP\x01Z>github.com/Iotic-Labs/iotic-go-proto-qapi/iotics/api;ioticsapi\xa2\x02\x03IAX\xaa\x02\nIotics.Api\xca\x02\nIotics\\Apib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'iotics.api.meta_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016com.iotics.apiB\tMetaProtoP\001Z>github.com/Iotic-Labs/iotic-go-proto-qapi/iotics/api;ioticsapi\242\002\003IAX\252\002\nIotics.Api\312\002\nIotics\\Api'
  _globals['_SPARQLRESULTTYPE']._serialized_start=953
  _globals['_SPARQLRESULTTYPE']._serialized_end=1067
  _globals['_SPARQLQUERYREQUEST']._serialized_start=88
  _globals['_SPARQLQUERYREQUEST']._serialized_end=371
  _globals['_SPARQLQUERYREQUEST_PAYLOAD']._serialized_start=264
  _globals['_SPARQLQUERYREQUEST_PAYLOAD']._serialized_end=371
  _globals['_SPARQLQUERYRESPONSE']._serialized_start=374
  _globals['_SPARQLQUERYRESPONSE']._serialized_end=707
  _globals['_SPARQLQUERYRESPONSE_PAYLOAD']._serialized_start=512
  _globals['_SPARQLQUERYRESPONSE_PAYLOAD']._serialized_end=707
  _globals['_SPARQLUPDATEREQUEST']._serialized_start=710
  _globals['_SPARQLUPDATEREQUEST']._serialized_end=880
  _globals['_SPARQLUPDATEREQUEST_PAYLOAD']._serialized_start=847
  _globals['_SPARQLUPDATEREQUEST_PAYLOAD']._serialized_end=880
  _globals['_SPARQLUPDATERESPONSE']._serialized_start=882
  _globals['_SPARQLUPDATERESPONSE']._serialized_end=951
  _globals['_METAAPI']._serialized_start=1070
  _globals['_METAAPI']._serialized_end=1248
# @@protoc_insertion_point(module_scope)



# Copyright © 2022 IOTIC LABS LTD. info@iotics.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://github.com/Iotic-Labs/iotics-grpc-client-py/blob/main/LICENSE
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import typing

import grpc
import grpc._channel

from .base import ApiBase
from .helpers import create_headers
from iotics.api.meta_pb2_grpc import MetaAPIStub as SparqlApiStub
from iotics.api import meta_pb2
from iotics.api import common_pb2


class SparqlApi(ApiBase):
    stub_class = SparqlApiStub
    stub: SparqlApiStub

    def sparql_query(
            self,
            query: str,
            result_content_type: meta_pb2.SparqlResultType = meta_pb2.SparqlResultType.SPARQL_JSON,
            scope: common_pb2.Scope = common_pb2.Scope.LOCAL,
            timeout: int = 5,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> meta_pb2.SparqlQueryResponse:
        """Submit a SPARQL query against the Federated Knowledge Graph of the Iotics network and receive a chunked stream of responses.

        Args:
            query: the SPARQL query to submit
            result_content_type: Choice of response type among SPARQL_JSON, SPARQL_XML, SPARQL_CSV, RDF_TURTLE, RDF_XML,
                and RDF_NTRIPLES
            scope: GLOBAL or LOCAL, whether to submit this query to only the local host or all hosts on the network
            timeout: How long to wait for responses, in seconds
            headers: optional request headers

        Returns: A result chunk with details of where it is in the sequence, and whether it is the last chunk
        """
        req = meta_pb2.SparqlQueryRequest(
            headers=headers or create_headers(),
            payload=meta_pb2.SparqlQueryRequest.Payload(
                query=query.encode('utf-8'),
                resultContentType=result_content_type
            ), scope=scope
        )
        # Define type here to avoid: `TypeError: 'ABCMeta' object is not subscriptable`.
        chunks: grpc._channel._MultiThreadedRendezvous[meta_pb2.SparqlQueryResponse] = \
            self.stub.SparqlQuery(req, timeout=timeout)
        return chunks

    def sparql_update(
            self,
            update: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> meta_pb2.SparqlUpdateResponse:
        """Add triples to the custom-public graph of the Fuseki database on the local host.

        Args:
            update: the update written in SPARQL
            headers: optional request headers

        Returns: Response object
        """
        req = meta_pb2.SparqlUpdateRequest(
            headers=headers or create_headers(),
            payload=meta_pb2.SparqlUpdateRequest.Payload(update=update.encode('utf-8'))
        )
        return self.stub.SparqlUpdate(req)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from iotics.api import common_pb2 as iotics_dot_api_dot_common__pb2
from iotics.api import search_pb2 as iotics_dot_api_dot_search__pb2


class SearchAPIStub(object):
    """---------------------------------------------------------------------------------------------------------------------

    SearchAPI provides a set of services to run synchronous and asynchronous search.
    Services only affect local resources, unless stated otherwise.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.DispatchSearchRequest = channel.unary_unary(
                '/iotics.api.SearchAPI/DispatchSearchRequest',
                request_serializer=iotics_dot_api_dot_search__pb2.SearchRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_search__pb2.DispatchSearchResponse.FromString,
                _registered_method=True)
        self.SynchronousSearch = channel.unary_stream(
                '/iotics.api.SearchAPI/SynchronousSearch',
                request_serializer=iotics_dot_api_dot_search__pb2.SearchRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_search__pb2.SearchResponse.FromString,
                _registered_method=True)
        self.ReceiveAllSearchResponses = channel.unary_stream(
                '/iotics.api.SearchAPI/ReceiveAllSearchResponses',
                request_serializer=iotics_dot_api_dot_common__pb2.SubscriptionHeaders.SerializeToString,
                response_deserializer=iotics_dot_api_dot_search__pb2.SearchResponse.FromString,
                _registered_method=True)
        self.AdvancedSearch = channel.unary_stream(
                '/iotics.api.SearchAPI/AdvancedSearch',
                request_serializer=iotics_dot_api_dot_search__pb2.AdvancedSearchRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_search__pb2.SearchResponse.FromString,
                _registered_method=True)


class SearchAPIServicer(object):
    """---------------------------------------------------------------------------------------------------------------------

    SearchAPI provides a set of services to run synchronous and asynchronous search.
    Services only affect local resources, unless stated otherwise.
    """

    def DispatchSearchRequest(self, request, context):
        """Send a search request. Results are expected asynchronously. (local and remote)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SynchronousSearch(self, request, context):
        """Run a synchronous search based on a user timeout. (local and remote)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReceiveAllSearchResponses(self, request, context):
        """Receive all search responses associated to a set of Search request for a given client application ID.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdvancedSearch(self, request, context):
        """Run an advanced (filter) search request on a user timeout and return formatted results.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SearchAPIServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'DispatchSearchRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.DispatchSearchRequest,
                    request_deserializer=iotics_dot_api_dot_search__pb2.SearchRequest.FromString,
                    response_serializer=iotics_dot_api_dot_search__pb2.DispatchSearchResponse.SerializeToString,
            ),
            'SynchronousSearch': grpc.unary_stream_rpc_method_handler(
                    servicer.SynchronousSearch,
                    request_deserializer=iotics_dot_api_dot_search__pb2.SearchRequest.FromString,
                    response_serializer=iotics_dot_api_dot_search__pb2.SearchResponse.SerializeToString,
            ),
            'ReceiveAllSearchResponses': grpc.unary_stream_rpc_method_handler(
                    servicer.ReceiveAllSearchResponses,
                    request_deserializer=iotics_dot_api_dot_common__pb2.SubscriptionHeaders.FromString,
                    response_serializer=iotics_dot_api_dot_search__pb2.SearchResponse.SerializeToString,
            ),
            'AdvancedSearch': grpc.unary_stream_rpc_method_handler(
                    servicer.AdvancedSearch,
                    request_deserializer=iotics_dot_api_dot_search__pb2.AdvancedSearchRequest.FromString,
                    response_serializer=iotics_dot_api_dot_search__pb2.SearchResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'iotics.api.SearchAPI', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('iotics.api.SearchAPI', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class SearchAPI(object):
    """---------------------------------------------------------------------------------------------------------------------

    SearchAPI provides a set of services to run synchronous and asynchronous search.
    Services only affect local resources, unless stated otherwise.
    """

    @staticmethod
    def DispatchSearchRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.SearchAPI/DispatchSearchRequest',
            iotics_dot_api_dot_search__pb2.SearchRequest.SerializeToString,
            iotics_dot_api_dot_search__pb2.DispatchSearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SynchronousSearch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/iotics.api.SearchAPI/SynchronousSearch',
            iotics_dot_api_dot_search__pb2.SearchRequest.SerializeToString,
            iotics_dot_api_dot_search__pb2.SearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReceiveAllSearchResponses(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/iotics.api.SearchAPI/ReceiveAllSearchResponses',
            iotics_dot_api_dot_common__pb2.SubscriptionHeaders.SerializeToString,
            iotics_dot_api_dot_search__pb2.SearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AdvancedSearch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/iotics.api.SearchAPI/AdvancedSearch',
            iotics_dot_api_dot_search__pb2.AdvancedSearchRequest.SerializeToString,
            iotics_dot_api_dot_search__pb2.SearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright (c) 2019-2024 Iotic Labs Ltd. All rights reserved.

Iotics Web protocol definitions (input)
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import iotics.api.common_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class InputID(google.protobuf.message.Message):
    """Representation of an input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    TWINID_FIELD_NUMBER: builtins.int
    HOSTID_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Input Identifier string representation (simple string) (unique within the scope of a twin identifier's input set)"""
    twinId: builtins.str
    """Twin identifier string representation (simple string) (twin to which the input belongs)"""
    hostId: builtins.str
    """Host identifier string representation (simple string) (Host to which the twin belongs)"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        twinId: builtins.str = ...,
        hostId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["hostId", b"hostId", "id", b"id", "twinId", b"twinId"]) -> None: ...

global___InputID = InputID

@typing.final
class DeleteInputRequest(google.protobuf.message.Message):
    """DeleteInputRequest is used to delete an input from a given twin."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Arguments(google.protobuf.message.Message):
        """DeleteInputRequest arguments."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Input to delete"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """DeleteInputRequest headers"""

    @property
    def args(self) -> global___DeleteInputRequest.Arguments:
        """DeleteInputRequest mandatory arguments"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___DeleteInputRequest.Arguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> None: ...

global___DeleteInputRequest = DeleteInputRequest

@typing.final
class DeleteInputResponse(google.protobuf.message.Message):
    """DeleteInputResponse describes a deleted input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        """DeleteInputResponse payload."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Deleted input"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """DeleteInputResponse headers"""

    @property
    def payload(self) -> global___DeleteInputResponse.Payload:
        """DeleteInputResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___DeleteInputResponse.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___DeleteInputResponse = DeleteInputResponse

@typing.final
class CreateInputRequest(google.protobuf.message.Message):
    """CreateInputRequest describes a created input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        """CreateInputRequest payload."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ID_FIELD_NUMBER: builtins.int
        id: builtins.str
        """Input identifier string representation (simple string) (unique within the scope of a twin identifier's input set)"""
        def __init__(
            self,
            *,
            id: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["id", b"id"]) -> None: ...

    @typing.final
    class Arguments(google.protobuf.message.Message):
        """Arguments describes the mandatory arguments to identify the twin owning this input"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TWINID_FIELD_NUMBER: builtins.int
        @property
        def twinId(self) -> iotics.api.common_pb2.TwinID:
            """Identifier of the twin owning this input"""

        def __init__(
            self,
            *,
            twinId: iotics.api.common_pb2.TwinID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["twinId", b"twinId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["twinId", b"twinId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """CreateInputRequest headers"""

    @property
    def args(self) -> global___CreateInputRequest.Arguments:
        """CreateInputRequest arguments"""

    @property
    def payload(self) -> global___CreateInputRequest.Payload:
        """CreateInputRequest payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___CreateInputRequest.Arguments | None = ...,
        payload: global___CreateInputRequest.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers", "payload", b"payload"]) -> None: ...

global___CreateInputRequest = CreateInputRequest

@typing.final
class CreateInputResponse(google.protobuf.message.Message):
    """CreateInputResponse describes a created input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        """CreateInputResponse payload."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Created input"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """CreateInputResponse headers"""

    @property
    def payload(self) -> global___CreateInputResponse.Payload:
        """CreateInputResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___CreateInputResponse.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___CreateInputResponse = CreateInputResponse

@typing.final
class UpdateInputRequest(google.protobuf.message.Message):
    """UpdateInputRequest is used to update an input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        """UdpateInputRequest payload used to update one or more field"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUES_FIELD_NUMBER: builtins.int
        PROPERTIES_FIELD_NUMBER: builtins.int
        @property
        def values(self) -> iotics.api.common_pb2.Values:
            """Values describe individual data items to add/remove"""

        @property
        def properties(self) -> iotics.api.common_pb2.PropertyUpdate:
            """Properties describe individual properties to add/remove. Note only custom properties can be modified here."""

        def __init__(
            self,
            *,
            values: iotics.api.common_pb2.Values | None = ...,
            properties: iotics.api.common_pb2.PropertyUpdate | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["properties", b"properties", "values", b"values"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["properties", b"properties", "values", b"values"]) -> None: ...

    @typing.final
    class Arguments(google.protobuf.message.Message):
        """UpdateInputRequest arguments"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Input to update"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """UpdateInputRequest headers"""

    @property
    def args(self) -> global___UpdateInputRequest.Arguments:
        """UpdateInputRequest arguments"""

    @property
    def payload(self) -> global___UpdateInputRequest.Payload:
        """UpdateInputRequest payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___UpdateInputRequest.Arguments | None = ...,
        payload: global___UpdateInputRequest.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers", "payload", b"payload"]) -> None: ...

global___UpdateInputRequest = UpdateInputRequest

@typing.final
class UpdateInputResponse(google.protobuf.message.Message):
    """UpdateInputResponse describes an updated input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        """UpdateInputResponse payload."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Updated input"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """UpdateInputResponse headers"""

    @property
    def payload(self) -> global___UpdateInputResponse.Payload:
        """UpdateInputResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___UpdateInputResponse.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___UpdateInputResponse = UpdateInputResponse

@typing.final
class DescribeInputRequest(google.protobuf.message.Message):
    """DescribeInputRequest is used to request the input metadata."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Arguments(google.protobuf.message.Message):
        """DescribeInputRequest arguments."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Input to describe"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """DescribeInputRequest headers"""

    @property
    def args(self) -> global___DescribeInputRequest.Arguments:
        """DescribeInputRequest mandatory arguments"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___DescribeInputRequest.Arguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> None: ...

global___DescribeInputRequest = DescribeInputRequest

@typing.final
class DescribeInputResponse(google.protobuf.message.Message):
    """DescribeInputResponse provides metadata lookup for individual input resources."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MetaResult(google.protobuf.message.Message):
        """DescribeInputResponse metadata result."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        VALUES_FIELD_NUMBER: builtins.int
        PROPERTIES_FIELD_NUMBER: builtins.int
        @property
        def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[iotics.api.common_pb2.Value]:
            """Values semantically describing the Input messages"""

        @property
        def properties(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[iotics.api.common_pb2.Property]:
            """Custom properties associated with this input."""

        def __init__(
            self,
            *,
            values: collections.abc.Iterable[iotics.api.common_pb2.Value] | None = ...,
            properties: collections.abc.Iterable[iotics.api.common_pb2.Property] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["properties", b"properties", "values", b"values"]) -> None: ...

    @typing.final
    class Payload(google.protobuf.message.Message):
        """DescribeInputResponse payload."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        RESULT_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Described input"""

        @property
        def result(self) -> global___DescribeInputResponse.MetaResult:
            """Metadata result"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
            result: global___DescribeInputResponse.MetaResult | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId", "result", b"result"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId", "result", b"result"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """DescribeInputResponse headers"""

    @property
    def payload(self) -> global___DescribeInputResponse.Payload:
        """DescribeInputResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___DescribeInputResponse.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___DescribeInputResponse = DescribeInputResponse

@typing.final
class UpsertInputWithMeta(google.protobuf.message.Message):
    """UpsertInputWithMeta is used to describe the full input state. Used in UpsertTwinRequest."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    VALUES_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Input Identifier string representation (simple string) (unique within the scope of a twin identifier's input set)"""
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[iotics.api.common_pb2.Value]:
        """Values to set"""

    @property
    def properties(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[iotics.api.common_pb2.Property]:
        """Properties to set"""

    def __init__(
        self,
        *,
        id: builtins.str = ...,
        values: collections.abc.Iterable[iotics.api.common_pb2.Value] | None = ...,
        properties: collections.abc.Iterable[iotics.api.common_pb2.Property] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id", "properties", b"properties", "values", b"values"]) -> None: ...

global___UpsertInputWithMeta = UpsertInputWithMeta

@typing.final
class InputMessage(google.protobuf.message.Message):
    """InputMessage describes a message that can be sent to an input"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OCCURREDAT_FIELD_NUMBER: builtins.int
    MIME_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    mime: builtins.str
    """Mime is the mime type of the encoded data."""
    data: builtins.bytes
    """Data is the actual message, encoded according the the mime type. The data should follow the Input's
    value definitions but that is not enforced. (See also Value)
    """
    @property
    def occurredAt(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """OccurredAt is the UTC timestamp of the message.
        Typically this is either the time at which an application sent this message
        or the time applicable to the message itself. (Optional: set to host time if not provided)
        """

    def __init__(
        self,
        *,
        occurredAt: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        mime: builtins.str = ...,
        data: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["occurredAt", b"occurredAt"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "mime", b"mime", "occurredAt", b"occurredAt"]) -> None: ...

global___InputMessage = InputMessage

@typing.final
class ReceiveInputMessageRequest(google.protobuf.message.Message):
    """ReceiveInputMessageRequest is used to receive messages sent to a given Input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Arguments(google.protobuf.message.Message):
        """ReceiveInputMessageRequest arguments."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Input to listen messages from"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """ReceiveInputMessageRequest headers"""

    @property
    def args(self) -> global___ReceiveInputMessageRequest.Arguments:
        """ReceiveInputMessageRequest mandatory arguments"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        args: global___ReceiveInputMessageRequest.Arguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["args", b"args", "headers", b"headers"]) -> None: ...

global___ReceiveInputMessageRequest = ReceiveInputMessageRequest

@typing.final
class ReceiveInputMessageResponse(google.protobuf.message.Message):
    """ReceiveInputMessageResponse contains a message sent to the Input."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Payload(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INPUTID_FIELD_NUMBER: builtins.int
        MESSAGE_FIELD_NUMBER: builtins.int
        @property
        def inputId(self) -> global___InputID:
            """Input the message has been sent to"""

        @property
        def message(self) -> global___InputMessage:
            """Input message"""

        def __init__(
            self,
            *,
            inputId: global___InputID | None = ...,
            message: global___InputMessage | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["inputId", b"inputId", "message", b"message"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["inputId", b"inputId", "message", b"message"]) -> None: ...

    HEADERS_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> iotics.api.common_pb2.Headers:
        """ReceiveInputMessageResponse headers"""

    @property
    def payload(self) -> global___ReceiveInputMessageResponse.Payload:
        """ReceiveInputMessageResponse payload"""

    def __init__(
        self,
        *,
        headers: iotics.api.common_pb2.Headers | None = ...,
        payload: global___ReceiveInputMessageResponse.Payload | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["headers", b"headers", "payload", b"payload"]) -> None: ...

global___ReceiveInputMessageResponse = ReceiveInputMessageResponse



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from iotics.api import host_pb2 as iotics_dot_api_dot_host__pb2


class HostAPIStub(object):
    """---------------------------------------------------------------------------------------------------------------------

    HostAPI enables management of Iotics host twins.
    Services only affect local resources, unless stated otherwise.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetHostID = channel.unary_unary(
                '/iotics.api.HostAPI/GetHostID',
                request_serializer=iotics_dot_api_dot_host__pb2.GetHostIDRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_host__pb2.GetHostIDResponse.FromString,
                _registered_method=True)
        self.IsHostDataAllowed = channel.unary_unary(
                '/iotics.api.HostAPI/IsHostDataAllowed',
                request_serializer=iotics_dot_api_dot_host__pb2.IsHostDataAllowedRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_host__pb2.IsHostDataAllowedResponse.FromString,
                _registered_method=True)
        self.IsHostMetaAllowed = channel.unary_unary(
                '/iotics.api.HostAPI/IsHostMetaAllowed',
                request_serializer=iotics_dot_api_dot_host__pb2.IsHostMetaAllowedRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_host__pb2.IsHostMetaAllowedResponse.FromString,
                _registered_method=True)


class HostAPIServicer(object):
    """---------------------------------------------------------------------------------------------------------------------

    HostAPI enables management of Iotics host twins.
    Services only affect local resources, unless stated otherwise.
    """

    def GetHostID(self, request, context):
        """GetHostID gets the ID of the host twin.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IsHostDataAllowed(self, request, context):
        """IsHostDataAllowed determines whether a remote host is allowed to perform data requests against the given twin.
        An example of a data request is interest.SendInputMessage
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IsHostMetaAllowed(self, request, context):
        """IsHostMetaAllowed determines whether a remote host is allowed to perform meta requests against the given twin.
        An example of a meta(data) request is twin.DescribeTwin
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HostAPIServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetHostID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetHostID,
                    request_deserializer=iotics_dot_api_dot_host__pb2.GetHostIDRequest.FromString,
                    response_serializer=iotics_dot_api_dot_host__pb2.GetHostIDResponse.SerializeToString,
            ),
            'IsHostDataAllowed': grpc.unary_unary_rpc_method_handler(
                    servicer.IsHostDataAllowed,
                    request_deserializer=iotics_dot_api_dot_host__pb2.IsHostDataAllowedRequest.FromString,
                    response_serializer=iotics_dot_api_dot_host__pb2.IsHostDataAllowedResponse.SerializeToString,
            ),
            'IsHostMetaAllowed': grpc.unary_unary_rpc_method_handler(
                    servicer.IsHostMetaAllowed,
                    request_deserializer=iotics_dot_api_dot_host__pb2.IsHostMetaAllowedRequest.FromString,
                    response_serializer=iotics_dot_api_dot_host__pb2.IsHostMetaAllowedResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'iotics.api.HostAPI', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('iotics.api.HostAPI', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class HostAPI(object):
    """---------------------------------------------------------------------------------------------------------------------

    HostAPI enables management of Iotics host twins.
    Services only affect local resources, unless stated otherwise.
    """

    @staticmethod
    def GetHostID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.HostAPI/GetHostID',
            iotics_dot_api_dot_host__pb2.GetHostIDRequest.SerializeToString,
            iotics_dot_api_dot_host__pb2.GetHostIDResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IsHostDataAllowed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.HostAPI/IsHostDataAllowed',
            iotics_dot_api_dot_host__pb2.IsHostDataAllowedRequest.SerializeToString,
            iotics_dot_api_dot_host__pb2.IsHostDataAllowedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IsHostMetaAllowed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.HostAPI/IsHostMetaAllowed',
            iotics_dot_api_dot_host__pb2.IsHostMetaAllowedRequest.SerializeToString,
            iotics_dot_api_dot_host__pb2.IsHostMetaAllowedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright (c) 2019-2024 Iotic Labs Ltd. All rights reserved.

Iotics Web protocol definitions (input)
"""

import abc
import collections.abc
import grpc
import grpc.aio
import iotics.api.input_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore[misc, type-arg]
    ...

class InputAPIStub:
    """---------------------------------------------------------------------------------------------------------------------

    Input API groups all the actions link to a twin input.
    Services only affect local resources, unless stated otherwise.
    """

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    DeleteInput: grpc.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.DeleteInputRequest,
        iotics.api.input_pb2.DeleteInputResponse,
    ]
    """Deletes an input. (Idempotent)"""

    DescribeInput: grpc.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.DescribeInputRequest,
        iotics.api.input_pb2.DescribeInputResponse,
    ]
    """Describes an input. (local and remote)"""

    ReceiveInputMessages: grpc.UnaryStreamMultiCallable[
        iotics.api.input_pb2.ReceiveInputMessageRequest,
        iotics.api.input_pb2.ReceiveInputMessageResponse,
    ]
    """Receives input messages for a specific input."""

    CreateInput: grpc.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.CreateInputRequest,
        iotics.api.input_pb2.CreateInputResponse,
    ]
    """Creates an input owned by a twin. (Idempotent)"""

    UpdateInput: grpc.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.UpdateInputRequest,
        iotics.api.input_pb2.UpdateInputResponse,
    ]
    """Updates attributes of an input, including its metadata."""

class InputAPIAsyncStub:
    """---------------------------------------------------------------------------------------------------------------------

    Input API groups all the actions link to a twin input.
    Services only affect local resources, unless stated otherwise.
    """

    DeleteInput: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.DeleteInputRequest,
        iotics.api.input_pb2.DeleteInputResponse,
    ]
    """Deletes an input. (Idempotent)"""

    DescribeInput: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.DescribeInputRequest,
        iotics.api.input_pb2.DescribeInputResponse,
    ]
    """Describes an input. (local and remote)"""

    ReceiveInputMessages: grpc.aio.UnaryStreamMultiCallable[
        iotics.api.input_pb2.ReceiveInputMessageRequest,
        iotics.api.input_pb2.ReceiveInputMessageResponse,
    ]
    """Receives input messages for a specific input."""

    CreateInput: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.CreateInputRequest,
        iotics.api.input_pb2.CreateInputResponse,
    ]
    """Creates an input owned by a twin. (Idempotent)"""

    UpdateInput: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.input_pb2.UpdateInputRequest,
        iotics.api.input_pb2.UpdateInputResponse,
    ]
    """Updates attributes of an input, including its metadata."""

class InputAPIServicer(metaclass=abc.ABCMeta):
    """---------------------------------------------------------------------------------------------------------------------

    Input API groups all the actions link to a twin input.
    Services only affect local resources, unless stated otherwise.
    """

    @abc.abstractmethod
    def DeleteInput(
        self,
        request: iotics.api.input_pb2.DeleteInputRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.input_pb2.DeleteInputResponse, collections.abc.Awaitable[iotics.api.input_pb2.DeleteInputResponse]]:
        """Deletes an input. (Idempotent)"""

    @abc.abstractmethod
    def DescribeInput(
        self,
        request: iotics.api.input_pb2.DescribeInputRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.input_pb2.DescribeInputResponse, collections.abc.Awaitable[iotics.api.input_pb2.DescribeInputResponse]]:
        """Describes an input. (local and remote)"""

    @abc.abstractmethod
    def ReceiveInputMessages(
        self,
        request: iotics.api.input_pb2.ReceiveInputMessageRequest,
        context: _ServicerContext,
    ) -> typing.Union[collections.abc.Iterator[iotics.api.input_pb2.ReceiveInputMessageResponse], collections.abc.AsyncIterator[iotics.api.input_pb2.ReceiveInputMessageResponse]]:
        """Receives input messages for a specific input."""

    @abc.abstractmethod
    def CreateInput(
        self,
        request: iotics.api.input_pb2.CreateInputRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.input_pb2.CreateInputResponse, collections.abc.Awaitable[iotics.api.input_pb2.CreateInputResponse]]:
        """Creates an input owned by a twin. (Idempotent)"""

    @abc.abstractmethod
    def UpdateInput(
        self,
        request: iotics.api.input_pb2.UpdateInputRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.input_pb2.UpdateInputResponse, collections.abc.Awaitable[iotics.api.input_pb2.UpdateInputResponse]]:
        """Updates attributes of an input, including its metadata."""

def add_InputAPIServicer_to_server(servicer: InputAPIServicer, server: typing.Union[grpc.Server, grpc.aio.Server]) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from iotics.api import feed_pb2 as iotics_dot_api_dot_feed__pb2


class FeedAPIStub(object):
    """---------------------------------------------------------------------------------------------------------------------

    Create a feed owned by the provided twin.
    A twin may have one or more feeds. Any twin can subscribe to a feed (access control permitting).
    A feed generates data in a 1-to-many relationship: one feed may produce data that is used by many consumers (twins).
    Services only affect local resources, unless stated otherwise.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateFeed = channel.unary_unary(
                '/iotics.api.FeedAPI/CreateFeed',
                request_serializer=iotics_dot_api_dot_feed__pb2.CreateFeedRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.CreateFeedResponse.FromString,
                _registered_method=True)
        self.DeleteFeed = channel.unary_unary(
                '/iotics.api.FeedAPI/DeleteFeed',
                request_serializer=iotics_dot_api_dot_feed__pb2.DeleteFeedRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.DeleteFeedResponse.FromString,
                _registered_method=True)
        self.UpdateFeed = channel.unary_unary(
                '/iotics.api.FeedAPI/UpdateFeed',
                request_serializer=iotics_dot_api_dot_feed__pb2.UpdateFeedRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.UpdateFeedResponse.FromString,
                _registered_method=True)
        self.ShareFeedData = channel.unary_unary(
                '/iotics.api.FeedAPI/ShareFeedData',
                request_serializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataResponse.FromString,
                _registered_method=True)
        self.StreamFeedData = channel.stream_unary(
                '/iotics.api.FeedAPI/StreamFeedData',
                request_serializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataResponse.FromString,
                _registered_method=True)
        self.ListAllFeeds = channel.unary_unary(
                '/iotics.api.FeedAPI/ListAllFeeds',
                request_serializer=iotics_dot_api_dot_feed__pb2.ListAllFeedsRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.ListAllFeedsResponse.FromString,
                _registered_method=True)
        self.DescribeFeed = channel.unary_unary(
                '/iotics.api.FeedAPI/DescribeFeed',
                request_serializer=iotics_dot_api_dot_feed__pb2.DescribeFeedRequest.SerializeToString,
                response_deserializer=iotics_dot_api_dot_feed__pb2.DescribeFeedResponse.FromString,
                _registered_method=True)


class FeedAPIServicer(object):
    """---------------------------------------------------------------------------------------------------------------------

    Create a feed owned by the provided twin.
    A twin may have one or more feeds. Any twin can subscribe to a feed (access control permitting).
    A feed generates data in a 1-to-many relationship: one feed may produce data that is used by many consumers (twins).
    Services only affect local resources, unless stated otherwise.
    """

    def CreateFeed(self, request, context):
        """Creates a feed owned by a twin. (Idempotent)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeed(self, request, context):
        """Deletes a feed owned by a twin. (Idempotent)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateFeed(self, request, context):
        """Updates attributes of a feed, including its metadata.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShareFeedData(self, request, context):
        """Shares a new sample of data for the given feed which any (interest) subscribers can receive.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamFeedData(self, request_iterator, context):
        """Shares feed data over a stream, any sharing error will return the error and close the stream.
        Note: cannot share to different feeds over the same stream.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListAllFeeds(self, request, context):
        """Lists all feeds owned by a twin.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeFeed(self, request, context):
        """Describes a feed. (local and remote)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FeedAPIServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateFeed': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateFeed,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.CreateFeedRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.CreateFeedResponse.SerializeToString,
            ),
            'DeleteFeed': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeed,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.DeleteFeedRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.DeleteFeedResponse.SerializeToString,
            ),
            'UpdateFeed': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateFeed,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.UpdateFeedRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.UpdateFeedResponse.SerializeToString,
            ),
            'ShareFeedData': grpc.unary_unary_rpc_method_handler(
                    servicer.ShareFeedData,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataResponse.SerializeToString,
            ),
            'StreamFeedData': grpc.stream_unary_rpc_method_handler(
                    servicer.StreamFeedData,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.ShareFeedDataResponse.SerializeToString,
            ),
            'ListAllFeeds': grpc.unary_unary_rpc_method_handler(
                    servicer.ListAllFeeds,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.ListAllFeedsRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.ListAllFeedsResponse.SerializeToString,
            ),
            'DescribeFeed': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeFeed,
                    request_deserializer=iotics_dot_api_dot_feed__pb2.DescribeFeedRequest.FromString,
                    response_serializer=iotics_dot_api_dot_feed__pb2.DescribeFeedResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'iotics.api.FeedAPI', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('iotics.api.FeedAPI', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class FeedAPI(object):
    """---------------------------------------------------------------------------------------------------------------------

    Create a feed owned by the provided twin.
    A twin may have one or more feeds. Any twin can subscribe to a feed (access control permitting).
    A feed generates data in a 1-to-many relationship: one feed may produce data that is used by many consumers (twins).
    Services only affect local resources, unless stated otherwise.
    """

    @staticmethod
    def CreateFeed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.FeedAPI/CreateFeed',
            iotics_dot_api_dot_feed__pb2.CreateFeedRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.CreateFeedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteFeed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.FeedAPI/DeleteFeed',
            iotics_dot_api_dot_feed__pb2.DeleteFeedRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.DeleteFeedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UpdateFeed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.FeedAPI/UpdateFeed',
            iotics_dot_api_dot_feed__pb2.UpdateFeedRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.UpdateFeedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ShareFeedData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.FeedAPI/ShareFeedData',
            iotics_dot_api_dot_feed__pb2.ShareFeedDataRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.ShareFeedDataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StreamFeedData(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(
            request_iterator,
            target,
            '/iotics.api.FeedAPI/StreamFeedData',
            iotics_dot_api_dot_feed__pb2.ShareFeedDataRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.ShareFeedDataResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListAllFeeds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.FeedAPI/ListAllFeeds',
            iotics_dot_api_dot_feed__pb2.ListAllFeedsRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.ListAllFeedsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DescribeFeed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/iotics.api.FeedAPI/DescribeFeed',
            iotics_dot_api_dot_feed__pb2.DescribeFeedRequest.SerializeToString,
            iotics_dot_api_dot_feed__pb2.DescribeFeedResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# Copyright © 2022 IOTIC LABS LTD. info@iotics.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://github.com/Iotic-Labs/iotics-grpc-client-py/blob/main/LICENSE
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import time
import typing

from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.wrappers_pb2 import BoolValue

from .base import ApiBase
from .helpers import create_headers
from iotics.api import feed_pb2_grpc
from iotics.api import feed_pb2
from iotics.api import common_pb2


class FeedApi(ApiBase):
    stub_class = feed_pb2_grpc.FeedAPIStub
    stub: feed_pb2_grpc.FeedAPIStub

    def create_feed(
            self, twin_did: str,
            feed_id: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> feed_pb2.CreateFeedResponse:
        """Create a feed on the given twin

        Args:
            twin_did: Decentralized Identifier of the twin on which a feed should be created
            feed_id: A string identifying this feed that is unique to this twin
            headers: optional request headers

        Returns: Response object confirming the feed's details (ie, the two IDs that were given)
        """
        req = feed_pb2.CreateFeedRequest(
            headers=headers or create_headers(),
            args=feed_pb2.CreateFeedRequest.Arguments(twinId=common_pb2.TwinID(id=twin_did)),
            payload=feed_pb2.CreateFeedRequest.Payload(id=feed_id)
        )
        return self.stub.CreateFeed(req)

    def delete_feed(
            self,
            twin_did: str,
            feed_id: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> feed_pb2.DeleteFeedResponse:
        """Delete a feed from the given twin

        Args:
            twin_did: Decentralized Identifier of the twin on which a feed should be created
            feed_id: A string identifying this feed that is unique to this twin
            headers: optional request headers

        Returns: Response object confirming the feed's details (ie, the two IDs that were given)
        """
        req = feed_pb2.DeleteFeedRequest(
            headers=headers or create_headers(),
            args=feed_pb2.DeleteFeedRequest.Arguments(feedId=feed_pb2.FeedID(id=feed_id, twinId=twin_did))
        )
        return self.stub.DeleteFeed(req)

    def update_feed(
            self,
            twin_did: str,
            feed_id: str,
            store_last: bool = True,
            values_added: typing.Optional[list] = None,
            values_deleted: typing.Optional[list] = None,
            props_added: typing.Optional[list] = None,
            props_deleted: typing.Optional[list] = None,
            props_keys_deleted: typing.Optional[list] = None,
            clear_all_props: bool = False,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> feed_pb2.UpdateFeedResponse:
        """Update the details of the given feed. Any arguments omitted will have their values unchanged

        Args:
            twin_did: Decentralized Identifier of the twin providing this feed
            feed_id: A string identifying this feed that is unique to this twin
            store_last: Whether the data last shared to this feed should be stored for retrieval via the Interest API,
                or only available to its subscribers at the moment it was shared
            values_added: A list of Value objects (metadata describing the structure of shared data) to be added to the
                feed. These may be created via the create_value helper
            values_deleted: A list of Value objects to be removed from the feed
            props_added: A list of semantic properties to be added to the feed
            props_deleted: A list of semantic properties to be removed from the feed
            props_keys_deleted: A list of any property keys for which all values should be removed from the feed
            clear_all_props: Whether or not to remove all semantic properties from the feed
            headers: optional request headers

        Returns: Response object confirming the IDs of the twin and feed being updated
        """
        req = feed_pb2.UpdateFeedRequest(
            headers=headers or create_headers(),
            args=feed_pb2.UpdateFeedRequest.Arguments(
                feedId=feed_pb2.FeedID(id=feed_id, twinId=twin_did)
            ),
            payload=feed_pb2.UpdateFeedRequest.Payload(
                storeLast=BoolValue(value=store_last),
                values=common_pb2.Values(added=values_added, deletedByLabel=values_deleted),
                properties=common_pb2.PropertyUpdate(
                    added=props_added, deleted=props_deleted, deletedByKey=props_keys_deleted,
                    clearedAll=clear_all_props
                )
            )
        )
        return self.stub.UpdateFeed(req)

    def share_feed_data(
            self,
            twin_did: str,
            feed_id: str,
            data: dict,
            occurred_at: typing.Optional[int] = None,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> feed_pb2.ShareFeedDataResponse:
        """Share data to a feed

        Args:
            twin_did: Decentralized Identifier of the twin providing the feed
            feed_id: A string identifying the feed that is unique to its twin
            data: A dictionary with keys matching labels of the metadata Value objects associated with this feed,
            and values representing their status at the given time
            occurred_at: The time at which this data was captured in seconds since the epoch, defaulting to now
            headers: optional request headers

        Returns: Response object bearing no payload
        """
        req = feed_pb2.ShareFeedDataRequest(
            headers=headers or create_headers(),
            args=feed_pb2.ShareFeedDataRequest.Arguments(
                feedId=feed_pb2.FeedID(id=feed_id, twinId=twin_did)
            ),
            payload=feed_pb2.ShareFeedDataRequest.Payload(sample=common_pb2.FeedData(
                occurredAt=Timestamp(seconds=occurred_at if occurred_at else int(time.time())),
                mime="application/json",
                data=bytes(json.dumps(data), 'utf-8'))),
        )
        return self.stub.ShareFeedData(req)

    def list_all_feeds(
            self,
            twin_did: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> feed_pb2.ListAllFeedsResponse:
        """List all feeds provided by a given twin

        Args:
            twin_did: Decentralized Identifier for the twin whose feeds are to be listed
            headers: optional request headers

        Returns: Response object with attribute 'feeds', a list of Feed objects with id and twinId attributes
        """
        req = feed_pb2.ListAllFeedsRequest(
            headers=headers or create_headers(),
            args=feed_pb2.ListAllFeedsRequest.Arguments(twinId=common_pb2.TwinID(id=twin_did))
        )
        return self.stub.ListAllFeeds(req)

    def describe_feed(
            self,
            twin_did: str,
            feed_id: str,
            remote_host_id: typing.Optional[str] = None,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> feed_pb2.DescribeFeedResponse:
        """
        Describes the feed identified by the given twin and feed IDs, listing its values, properties, and whether its
            last shared data is stored
        Args:
            twin_did: Decentralized Identifier for the twin exposing the feed in question
            feed_id: A string identifying the feed that is unique to its twin
            remote_host_id: ID of the remote host on which the twin can be found (None if local)
            headers: optional request headers

        Returns: Response object describing the feed
        """
        req = feed_pb2.DescribeFeedRequest(
            headers=headers or create_headers(),
            args=feed_pb2.DescribeFeedRequest.Arguments(
                feedId=feed_pb2.FeedID(id=feed_id, twinId=twin_did, hostId=remote_host_id)),
        )
        return self.stub.DescribeFeed(req)



# Copyright © 2024 IOTIC LABS LTD. info@iotics.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://github.com/Iotic-Labs/iotics-grpc-client-py/blob/main/LICENSE
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import typing

from iotics.api import common_pb2
from iotics.api import circle_pb2
from iotics.api import circle_pb2_grpc
from .base import ApiBase
from .helpers import create_headers, PER_PAGE_LIMIT


class CircleAPI(ApiBase):
    stub_class = circle_pb2_grpc.CircleAPIStub
    stub: circle_pb2_grpc.CircleAPIStub

    def describe_circle(
            self,
            circle_did: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> circle_pb2.DescribeCircleResponse:
        """Describes the circle with the given DID, listing its properties

        Args:
            circle_did: Decentralized Identifier uniquely specifying the circle
            headers: optional request headers

        Returns: Response object describing the circle
        """

        req = circle_pb2.DescribeCircleRequest(
            headers=headers or create_headers(),
            args=circle_pb2.DescribeCircleRequest.Arguments(
                circleId=circle_pb2.CircleID(id=circle_did)),
        )
        return self.stub.DescribeCircle(req)

    def delete_circle(
            self,
            circle_did: str,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> circle_pb2.DeleteCircleResponse:
        """Deletes the circle with the given DID

        Args:
            circle_did: Decentralized Identifier uniquely specifying the circle
            headers: optional request headers

        Returns: Response object confirming the DID of the deleted circle
        """

        req = circle_pb2.DeleteCircleRequest(
            headers=headers or create_headers(),
            args=circle_pb2.DeleteCircleRequest.Arguments(circleId=circle_pb2.CircleID(id=circle_did)))
        return self.stub.DeleteCircle(req)

    def list_circles(self,
                   page:int = 0,
                   headers: typing.Optional[common_pb2.Headers] = None
    ) -> circle_pb2.ListAllCirclesResponse:
        """Lists all local circles visible to the user making the request

        Args:
            page: Used to request offsetted results, as responses contain only up to `PER_PAGE_LIMIT` results per request.
            headers: optional request headers

        Returns: Response object listing circles with their location and properties
        """
        
        req = circle_pb2.ListAllCirclesRequest(
            range=common_pb2.Range(
                limit=common_pb2.Limit(value=PER_PAGE_LIMIT),
                offset=common_pb2.Offset(value=PER_PAGE_LIMIT * page)
            ),
            headers=headers or create_headers())
        return self.stub.ListAllCircles(req)

    def upsert_circle(
            self,
            circle_did: str,
            properties: typing.Optional[typing.Iterable[common_pb2.Property]] = None,
            headers: typing.Optional[common_pb2.Headers] = None
    ) -> circle_pb2.UpsertCircleResponse:
        """Upserts a circle with the given details, ie, the given ID will specify a circle with these details regardless of
        whether a circle previously existed with this ID. Any arguments omitted will become absent or given default values

        Args:
            circle_did: Decentralized Identifier of the circle to upsert
            properties: A list of semantic properties providing further information about the circle, created using the
                create_property helper
            headers: optional request headers

        Returns: Response object confirming the ID of the circle that was upserted
        """

        req = circle_pb2.UpsertCircleRequest(
            headers=headers or create_headers(),
            payload=circle_pb2.UpsertCircleRequest.Payload(
                circleId=circle_pb2.CircleID(id=circle_did),
                properties=properties,
            )
        )
        return self.stub.UpsertCircle(req)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright (c) 2019-2024 Iotic Labs Ltd. All rights reserved.

Iotics Web protocol definitions (feed)
"""

import abc
import collections.abc
import grpc
import grpc.aio
import iotics.api.feed_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore[misc, type-arg]
    ...

class FeedAPIStub:
    """---------------------------------------------------------------------------------------------------------------------

    Create a feed owned by the provided twin.
    A twin may have one or more feeds. Any twin can subscribe to a feed (access control permitting).
    A feed generates data in a 1-to-many relationship: one feed may produce data that is used by many consumers (twins).
    Services only affect local resources, unless stated otherwise.
    """

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    CreateFeed: grpc.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.CreateFeedRequest,
        iotics.api.feed_pb2.CreateFeedResponse,
    ]
    """Creates a feed owned by a twin. (Idempotent)"""

    DeleteFeed: grpc.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.DeleteFeedRequest,
        iotics.api.feed_pb2.DeleteFeedResponse,
    ]
    """Deletes a feed owned by a twin. (Idempotent)"""

    UpdateFeed: grpc.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.UpdateFeedRequest,
        iotics.api.feed_pb2.UpdateFeedResponse,
    ]
    """Updates attributes of a feed, including its metadata."""

    ShareFeedData: grpc.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.ShareFeedDataRequest,
        iotics.api.feed_pb2.ShareFeedDataResponse,
    ]
    """Shares a new sample of data for the given feed which any (interest) subscribers can receive."""

    StreamFeedData: grpc.StreamUnaryMultiCallable[
        iotics.api.feed_pb2.ShareFeedDataRequest,
        iotics.api.feed_pb2.ShareFeedDataResponse,
    ]
    """Shares feed data over a stream, any sharing error will return the error and close the stream.
    Note: cannot share to different feeds over the same stream.
    """

    ListAllFeeds: grpc.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.ListAllFeedsRequest,
        iotics.api.feed_pb2.ListAllFeedsResponse,
    ]
    """Lists all feeds owned by a twin."""

    DescribeFeed: grpc.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.DescribeFeedRequest,
        iotics.api.feed_pb2.DescribeFeedResponse,
    ]
    """Describes a feed. (local and remote)"""

class FeedAPIAsyncStub:
    """---------------------------------------------------------------------------------------------------------------------

    Create a feed owned by the provided twin.
    A twin may have one or more feeds. Any twin can subscribe to a feed (access control permitting).
    A feed generates data in a 1-to-many relationship: one feed may produce data that is used by many consumers (twins).
    Services only affect local resources, unless stated otherwise.
    """

    CreateFeed: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.CreateFeedRequest,
        iotics.api.feed_pb2.CreateFeedResponse,
    ]
    """Creates a feed owned by a twin. (Idempotent)"""

    DeleteFeed: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.DeleteFeedRequest,
        iotics.api.feed_pb2.DeleteFeedResponse,
    ]
    """Deletes a feed owned by a twin. (Idempotent)"""

    UpdateFeed: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.UpdateFeedRequest,
        iotics.api.feed_pb2.UpdateFeedResponse,
    ]
    """Updates attributes of a feed, including its metadata."""

    ShareFeedData: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.ShareFeedDataRequest,
        iotics.api.feed_pb2.ShareFeedDataResponse,
    ]
    """Shares a new sample of data for the given feed which any (interest) subscribers can receive."""

    StreamFeedData: grpc.aio.StreamUnaryMultiCallable[
        iotics.api.feed_pb2.ShareFeedDataRequest,
        iotics.api.feed_pb2.ShareFeedDataResponse,
    ]
    """Shares feed data over a stream, any sharing error will return the error and close the stream.
    Note: cannot share to different feeds over the same stream.
    """

    ListAllFeeds: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.ListAllFeedsRequest,
        iotics.api.feed_pb2.ListAllFeedsResponse,
    ]
    """Lists all feeds owned by a twin."""

    DescribeFeed: grpc.aio.UnaryUnaryMultiCallable[
        iotics.api.feed_pb2.DescribeFeedRequest,
        iotics.api.feed_pb2.DescribeFeedResponse,
    ]
    """Describes a feed. (local and remote)"""

class FeedAPIServicer(metaclass=abc.ABCMeta):
    """---------------------------------------------------------------------------------------------------------------------

    Create a feed owned by the provided twin.
    A twin may have one or more feeds. Any twin can subscribe to a feed (access control permitting).
    A feed generates data in a 1-to-many relationship: one feed may produce data that is used by many consumers (twins).
    Services only affect local resources, unless stated otherwise.
    """

    @abc.abstractmethod
    def CreateFeed(
        self,
        request: iotics.api.feed_pb2.CreateFeedRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.CreateFeedResponse, collections.abc.Awaitable[iotics.api.feed_pb2.CreateFeedResponse]]:
        """Creates a feed owned by a twin. (Idempotent)"""

    @abc.abstractmethod
    def DeleteFeed(
        self,
        request: iotics.api.feed_pb2.DeleteFeedRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.DeleteFeedResponse, collections.abc.Awaitable[iotics.api.feed_pb2.DeleteFeedResponse]]:
        """Deletes a feed owned by a twin. (Idempotent)"""

    @abc.abstractmethod
    def UpdateFeed(
        self,
        request: iotics.api.feed_pb2.UpdateFeedRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.UpdateFeedResponse, collections.abc.Awaitable[iotics.api.feed_pb2.UpdateFeedResponse]]:
        """Updates attributes of a feed, including its metadata."""

    @abc.abstractmethod
    def ShareFeedData(
        self,
        request: iotics.api.feed_pb2.ShareFeedDataRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.ShareFeedDataResponse, collections.abc.Awaitable[iotics.api.feed_pb2.ShareFeedDataResponse]]:
        """Shares a new sample of data for the given feed which any (interest) subscribers can receive."""

    @abc.abstractmethod
    def StreamFeedData(
        self,
        request_iterator: _MaybeAsyncIterator[iotics.api.feed_pb2.ShareFeedDataRequest],
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.ShareFeedDataResponse, collections.abc.Awaitable[iotics.api.feed_pb2.ShareFeedDataResponse]]:
        """Shares feed data over a stream, any sharing error will return the error and close the stream.
        Note: cannot share to different feeds over the same stream.
        """

    @abc.abstractmethod
    def ListAllFeeds(
        self,
        request: iotics.api.feed_pb2.ListAllFeedsRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.ListAllFeedsResponse, collections.abc.Awaitable[iotics.api.feed_pb2.ListAllFeedsResponse]]:
        """Lists all feeds owned by a twin."""

    @abc.abstractmethod
    def DescribeFeed(
        self,
        request: iotics.api.feed_pb2.DescribeFeedRequest,
        context: _ServicerContext,
    ) -> typing.Union[iotics.api.feed_pb2.DescribeFeedResponse, collections.abc.Awaitable[iotics.api.feed_pb2.DescribeFeedResponse]]:
        """Describes a feed. (local and remote)"""

def add_FeedAPIServicer_to_server(servicer: FeedAPIServicer, server: typing.Union[grpc.Server, grpc.aio.Server]) -> None: ...
